#!/usr/bin/perl

use strict;
use lib qw(.);

use Test::More;
use Data::Dumper;

BEGIN {
	eval "use DBD::SQLite";
	plan $@ ? (skip_all => 'needs DBD::SQLite for testing') : (tests => 16);
}

package My::DBI;

use base 'Class::DBI';

use File::Temp qw/tempfile/;
my (undef, $DB) = tempfile();
my @DSN = ("dbi:SQLite:dbname=$DB", '', '', { AutoCommit => 1 });
__PACKAGE__->set_db(Main => @DSN);

END { unlink $DB if -e $DB }


package My::Movie;

use base 'My::DBI';

use Class::DBI::Relationship::HasDetails;

__PACKAGE__->table('Movies');
__PACKAGE__->columns(All => qw/id title year/);

__PACKAGE__->has_details('details', 'My::MovieDetails' => 'movie');
__PACKAGE__->has_details('extra_details', 'My::MovieExtraDetails' => 'movie');

__PACKAGE__->details_columns(qw(tagline genre));
__PACKAGE__->extra_details_columns(qw(rating price format));

sub CONSTRUCT {
	shift->db_Main->do(
		qq{
     CREATE TABLE Movies (
        id     INTEGER PRIMARY KEY,
        title  VARCHAR(255),
        year   INTEGER
     )
	}
	);
}



package My::MovieDetails;

use base 'My::DBI';

__PACKAGE__->table('MovieDetails');
__PACKAGE__->columns(Essential => qw/id movie field value/);

sub CONSTRUCT {
	shift->db_Main->do(
		qq{
     			CREATE TABLE MovieDetails (
     				id     INTEGER PRIMARY KEY,
     				movie  INTEGER,
     				field  VARCHAR(255),
     				value  VARCHAR(1024)
     			)
		}
  	);
}

package My::MovieExtraDetails;

use base 'My::DBI';

__PACKAGE__->table('MovieExtraDetails');
__PACKAGE__->columns(Essential => qw/id movie field value/);

sub CONSTRUCT {
	shift->db_Main->do(
		qq{
     			CREATE TABLE MovieExtraDetails (
     				id     INTEGER PRIMARY KEY,
     				movie  INTEGER,
     				field  VARCHAR(255),
     				value  VARCHAR(1024)
     			)
		}
  	);
}




package main;

My::Movie->CONSTRUCT;
My::MovieDetails->CONSTRUCT;
My::MovieExtraDetails->CONSTRUCT;

my %films = (
	'Trainspotting' => 1996,
	'Se7en' => 1995,
);

while (my ($title, $year) = each %films) {
	My::Movie->create({ title => $title, year => $year });
}


{
	my @films = My::Movie->retrieve_all;
	is @films, 2, "Got 2 films";
	$films[0]->title;
	$films[1]->title;
#	warn('dto: ' . $films[0]->__details_objects);
#	warn(Dumper(@films));
}

{
	my @columns = My::Movie->details_columns;
	is @columns, 2, "Got 2 details columns";

	ok(eq_set(\@columns, ['tagline', 'genre']), 'Details columns match');
}

{
	my @columns = My::Movie->extra_details_columns;
	is @columns, 3, "Got 3 extra_details columns";
	
	ok(eq_set(\@columns, ['rating', 'price', 'format']), 'Extra details columns match');
}

{
	my $film = My::Movie->search('title' => 'Trainspotting')->first;
#	warn(Dumper($film));

	$film->genre('Drama');
	$film->tagline('Choose life.');
	$film->price('$15.99');
	$film->update;
	is($film->genre, 'Drama', "Film type is Drama before re-read");

}

{
	my @details = My::MovieDetails->retrieve_all;
	is @details, 2, "Got 2 stored MovieDetails";
}

{
	my $ok = 0;

	my $detail = My::MovieDetails->search('field' => 'genre')->first;
	$detail->value eq 'Drama' and 
		$ok++;

	$detail = My::MovieDetails->search('field' => 'tagline')->first;
	$detail->value eq 'Choose life.' and
		$ok++;
	
	is($ok, 2, "MovieDetails fields and values match");
}



{
	my @details = My::MovieExtraDetails->retrieve_all;
	is @details, 1, "Got 1 stored MovieExtraDetails";

	ok($details[0]->field eq 'price' && $details[0]->value eq '$15.99', "MovieExtraDetails fields and values match");
}


{
	my $film = My::Movie->search('title' => 'Trainspotting')->first;
	is($film->genre, 'Drama', "Film type is Drama after re-read");
}


{
	my $film = My::Movie->search('title' => 'Se7en')->first;

	$film->genre('Thriller');
	$film->tagline('Seven deadly sins. Seven ways to die.');
	$film->price('$7.77');
	$film->update;
}

{
	my @details = My::MovieDetails->retrieve_all;
	is @details, 4, "Got 4 stored MovieDetails";
}

{
	my $film = My::Movie->search('title' => 'Se7en')->first;
	$film->genre('Mystery');
	$film->update;
}

{
	my $film = My::Movie->search('title' => 'Se7en')->first;
	my $detail = My::MovieDetails->search('movie' => $film->id, 'field' => 'genre')->first;
	is($detail->value, 'Mystery', "Updated MoveDetail ok");
}

{
	My::Movie->search('title' => 'Trainspotting')->delete_all;
	
	my @details = My::MovieDetails->retrieve_all;
	is @details, 2, "Got 4 stored MovieDetails after delete.";

	@details = My::MovieExtraDetails->retrieve_all;
	is @details, 1, "Got 1 stored MovieExtraDetails after delete.";
}

{
	my $film = My::Movie->search('title' => 'Se7en')->first;
	$film->delete_tagline();
	$film->update();

	my @details = My::MovieDetails->retrieve_all;
	is @details, 1, "Delete Details column";
}	
	
